@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2011 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2009-2012 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  XML parsing based on Xfce4 Panel:
  *  Copyright (c) 2005 Jasper Huijsmans <jasper@xfce.org>
@@ -36,8 +36,8 @@
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "common.h"
 
@@ -114,7 +114,7 @@ static void             end_element_handler                 (GMarkupParseContext
                                                              GError **error);
 static void             text_handler                        (GMarkupParseContext *context,
                                                              const gchar *text,
-                                                             gsize text_len,  
+                                                             gsize text_len,
                                                              gpointer user_data,
                                                              GError **error);
 
@@ -302,7 +302,7 @@ end_element_handler (GMarkupParseContext *context,
 static void
 text_handler (GMarkupParseContext *context,
               const gchar *text,
-              gsize text_len,  
+              gsize text_len,
               gpointer user_data,
               GError **error)
 {
@@ -432,6 +432,7 @@ static void
 __clipman_actions_entry_free (ClipmanActionsEntry *entry)
 {
   g_free (entry->action_name);
+  g_free (entry->pattern);
   g_regex_unref (entry->regex);
   g_hash_table_destroy (entry->commands);
   g_slice_free (ClipmanActionsEntry, entry);
@@ -686,12 +687,29 @@ clipman_actions_match_with_menu (ClipmanActions *actions,
   ClipmanActionsEntry *entry;
   GtkWidget *mi;
   GSList *l, *entries;
-  GdkModifierType state;
+  GdkModifierType state = 0;
+  GdkDisplay* display = gdk_display_get_default ();
+#if GTK_CHECK_VERSION (3, 20, 0)
+  GdkSeat *seat = gdk_display_get_default_seat (display);
+  GdkDevice *device = gdk_seat_get_pointer (seat);
+#else
+  GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
+  GdkDevice *device = gdk_device_manager_get_client_pointer (device_manager);
+#endif
+  GdkScreen* screen = gdk_screen_get_default ();
+  GdkWindow * root_win = gdk_screen_get_root_window (screen);
 
-  if (actions->priv->skip_action_on_key_down && group == ACTION_GROUP_SELECTION)
+  if (group == ACTION_GROUP_SELECTION)
     {
-      gdk_window_get_pointer (NULL, NULL, NULL, &state);
-      if (state & GDK_CONTROL_MASK)
+      gint ctrl_mask = 0;
+
+      gdk_window_get_device_position (root_win, device, NULL, NULL, &state);
+      ctrl_mask = state & GDK_CONTROL_MASK;
+      if (ctrl_mask && actions->priv->skip_action_on_key_down)
+        {
+          return;
+        }
+      else if (!ctrl_mask && !actions->priv->skip_action_on_key_down)
         {
           return;
         }
@@ -747,6 +765,13 @@ clipman_actions_match_with_menu (ClipmanActions *actions,
   gtk_container_add (GTK_CONTAINER (actions->priv->menu), mi);
 
   gtk_widget_show_all (actions->priv->menu);
+
+  if(!gtk_widget_has_grab(actions->priv->menu))
+  {
+    gtk_grab_add(actions->priv->menu);
+  }
+
+
   gtk_menu_popup (GTK_MENU (actions->priv->menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time ());
 
   g_slist_free (entries);
@@ -771,6 +796,13 @@ clipman_actions_load (ClipmanActions *actions)
 
   if (!load)
     {
+      /* Create user directory early to be sure it exists for next actions */
+      GFile *dir = g_file_get_parent (actions->priv->file);
+      g_file_make_directory_with_parents (dir, NULL, NULL);
+      g_object_unref (dir);
+      dir = NULL;
+
+      /* Load from system wide file */
       filename = g_strdup (SYSCONFDIR"/xdg/xfce4/panel/xfce4-clipman-actions.xml");
       load = g_file_get_contents (filename, &data, (gsize*)&size, NULL);
       g_free (filename);
@@ -982,4 +1014,3 @@ clipman_actions_get_property (GObject *object,
       break;
     }
 }
-
