@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008-2011 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2008-2012 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 /*
  * Status Icon
  */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 static MyPlugin*        status_icon_register            ();
 static gboolean         cb_status_icon_is_embedded      (GtkStatusIcon *status_icon);
@@ -89,9 +90,9 @@ status_icon_register (void)
     }
   else
     {
-      plugin->status_icon = gtk_status_icon_new_from_stock (GTK_STOCK_PASTE);
+      plugin->status_icon = gtk_status_icon_new_from_icon_name ("edit-paste");
     }
-  gtk_status_icon_set_tooltip (plugin->status_icon, _("Clipman"));
+  //gtk_status_icon_set_tooltip (plugin->status_icon, _("Clipman"));
   g_timeout_add_seconds (60, (GSourceFunc)cb_status_icon_is_embedded, plugin->status_icon);
 
   /* Signals */
@@ -133,6 +134,10 @@ cb_status_icon_popup_menu (MyPlugin *plugin, guint button, guint activate_time)
       mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
       gtk_menu_shell_append (GTK_MENU_SHELL (plugin->popup_menu), mi);
       g_signal_connect_swapped (mi, "activate", G_CALLBACK (plugin_configure), plugin);
+      mi = gtk_check_menu_item_new_with_mnemonic (_("_Disable"));
+      gtk_menu_shell_append (GTK_MENU_SHELL (plugin->popup_menu), mi);
+      xfconf_g_property_bind (plugin->channel, "/tweaks/inhibit",
+                              G_TYPE_BOOLEAN, mi, "active");
       mi = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
       gtk_menu_shell_append (GTK_MENU_SHELL (plugin->popup_menu), mi);
       g_signal_connect_swapped (mi, "activate", G_CALLBACK (plugin_about), plugin);
@@ -145,6 +150,12 @@ cb_status_icon_popup_menu (MyPlugin *plugin, guint button, guint activate_time)
     }
 
   gtk_menu_set_screen (GTK_MENU (plugin->popup_menu), gtk_status_icon_get_screen (plugin->status_icon));
+
+  if(!gtk_widget_has_grab(plugin->popup_menu))
+  {
+    gtk_grab_add(plugin->popup_menu);
+  }
+
   gtk_menu_popup (GTK_MENU (plugin->popup_menu), NULL, NULL,
                   (GtkMenuPositionFunc)gtk_status_icon_position_menu, plugin->status_icon,
                   0, gtk_get_current_event_time ());
@@ -232,4 +243,4 @@ update_autostart_file (gboolean autostart)
   g_key_file_free (keyfile);
   g_free (userfile);
 }
-
+G_GNUC_END_IGNORE_DEPRECATIONS
